'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _firebase = require('firebase');

var _firebase2 = _interopRequireDefault(_firebase);

var _reduxSaga = require('redux-saga');

var _effects = require('redux-saga/effects');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _marked = [createUserWithEmailAndPassword, signInWithEmailAndPassword, signInAnonymously, signInWithCustomToken, signInWithPopup, signInWithGoogle, signInWithFacebook, signInWithTwitter, signInWithGithub, signOut, syncUser].map(_regenerator2.default.mark); /**
                                                                                                                                                                                                                                                                      * A module for firebaseSagas.auth
                                                                                                                                                                                                                                                                      * @module auth
                                                                                                                                                                                                                                                                      */


/**
 * Creates a new user account associated with the specified email address and password.
 *
 * @param {string} email
 * @param {string} password
 * @returns {firebase.User} user
 */
function createUserWithEmailAndPassword(email, password) {
  var auth;
  return _regenerator2.default.wrap(function createUserWithEmailAndPassword$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          auth = this.app.auth();
          _context.next = 3;
          return (0, _effects.call)([auth, auth.createUserWithEmailAndPassword], email, password);

        case 3:
          return _context.abrupt('return', _context.sent);

        case 4:
        case 'end':
          return _context.stop();
      }
    }
  }, _marked[0], this);
}

/**
 * Signs in using an email and password.
 *
 * @param {string} email
 * @param {string} password
 * @returns {firebase.User} user
 */
function signInWithEmailAndPassword(email, password) {
  var auth;
  return _regenerator2.default.wrap(function signInWithEmailAndPassword$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          auth = this.app.auth();
          _context2.next = 3;
          return (0, _effects.call)([auth, auth.signInWithEmailAndPassword], email, password);

        case 3:
          return _context2.abrupt('return', _context2.sent);

        case 4:
        case 'end':
          return _context2.stop();
      }
    }
  }, _marked[1], this);
}

/**
 * Signs in as an anonymous user.
 *
 * @returns {firebase.User} user
 */
function signInAnonymously() {
  var auth;
  return _regenerator2.default.wrap(function signInAnonymously$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          auth = this.app.auth();
          _context3.next = 3;
          return (0, _effects.call)([auth, auth.signInAnonymously]);

        case 3:
          return _context3.abrupt('return', _context3.sent);

        case 4:
        case 'end':
          return _context3.stop();
      }
    }
  }, _marked[2], this);
}

/**
 * Signs in using a custom token.
 *
 * @param {string} token The custom token to sign in with.
 * @returns {firebase.User} user
 */
function signInWithCustomToken(token) {
  var auth;
  return _regenerator2.default.wrap(function signInWithCustomToken$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          auth = this.app.auth();
          _context4.next = 3;
          return (0, _effects.call)([auth, auth.signInWithCustomToken], token);

        case 3:
          return _context4.abrupt('return', _context4.sent);

        case 4:
        case 'end':
          return _context4.stop();
      }
    }
  }, _marked[3], this);
}

function signInWithPopup(provider, scopes, customParameters) {
  var auth, _ref, user;

  return _regenerator2.default.wrap(function signInWithPopup$(_context5) {
    while (1) {
      switch (_context5.prev = _context5.next) {
        case 0:
          auth = this.app.auth();

          if (scopes) {
            scopes.map(function (scope) {
              return provider.addScope(scope);
            });
          }
          if (customParameters) {
            provider.setCustomParameters(customParameters);
          }
          _context5.next = 5;
          return (0, _effects.call)([auth, auth.signInWithPopup], provider);

        case 5:
          _ref = _context5.sent;
          user = _ref.user;
          return _context5.abrupt('return', user);

        case 8:
        case 'end':
          return _context5.stop();
      }
    }
  }, _marked[4], this);
}

/**
 * Signs in using GoogleAuthProvider.
 *
 * @param {array} scopes Google OAuth scopes
 * @param {object} customParameters The custom OAuth parameters to pass in the OAuth request
 * @returns {firebase.User} user
 */
function signInWithGoogle(scopes, customParameters) {
  var provider;
  return _regenerator2.default.wrap(function signInWithGoogle$(_context6) {
    while (1) {
      switch (_context6.prev = _context6.next) {
        case 0:
          provider = new _firebase2.default.auth.GoogleAuthProvider();
          _context6.next = 3;
          return this.auth.signInWithPopup(provider, scopes, customParameters);

        case 3:
          return _context6.abrupt('return', _context6.sent);

        case 4:
        case 'end':
          return _context6.stop();
      }
    }
  }, _marked[5], this);
}

/**
 * Signs in using FacebookAuthProvider.
 *
 * @param {array} scopes Facebook OAuth scopes
 * @param {object} customParameters The custom OAuth parameters to pass in the OAuth request
 * @returns {firebase.User} user
 */
function signInWithFacebook(scopes, customParameters) {
  var provider;
  return _regenerator2.default.wrap(function signInWithFacebook$(_context7) {
    while (1) {
      switch (_context7.prev = _context7.next) {
        case 0:
          provider = new _firebase2.default.auth.FacebookAuthProvider();
          _context7.next = 3;
          return this.auth.signInWithPopup(provider, scopes, customParameters);

        case 3:
          return _context7.abrupt('return', _context7.sent);

        case 4:
        case 'end':
          return _context7.stop();
      }
    }
  }, _marked[6], this);
}

/**
 * Signs in using TwitterAuthProvider.
 *
 * @param {array} scopes Twitter OAuth scopes
 * @param {object} customParameters The custom OAuth parameters to pass in the OAuth request
 * @returns {firebase.User} user
 */
function signInWithTwitter(scopes, customParameters) {
  var provider;
  return _regenerator2.default.wrap(function signInWithTwitter$(_context8) {
    while (1) {
      switch (_context8.prev = _context8.next) {
        case 0:
          provider = new _firebase2.default.auth.TwitterAuthProvider();
          _context8.next = 3;
          return this.auth.signInWithPopup(provider, scopes, customParameters);

        case 3:
          return _context8.abrupt('return', _context8.sent);

        case 4:
        case 'end':
          return _context8.stop();
      }
    }
  }, _marked[7], this);
}

/**
 * Signs in using GithubAuthProvider.
 *
 * @param {array} scopes Github OAuth scopes
 * @param {object} customParameters The custom OAuth parameters to pass in the OAuth request
 * @returns {firebase.User} user
 */
function signInWithGithub(scopes, customParameters) {
  var provider;
  return _regenerator2.default.wrap(function signInWithGithub$(_context9) {
    while (1) {
      switch (_context9.prev = _context9.next) {
        case 0:
          provider = new _firebase2.default.auth.GithubAuthProvider();
          _context9.next = 3;
          return this.auth.signInWithPopup(provider, scopes, customParameters);

        case 3:
          return _context9.abrupt('return', _context9.sent);

        case 4:
        case 'end':
          return _context9.stop();
      }
    }
  }, _marked[8], this);
}

/**
 * Signs out the current user.
 */
function signOut() {
  var auth;
  return _regenerator2.default.wrap(function signOut$(_context10) {
    while (1) {
      switch (_context10.prev = _context10.next) {
        case 0:
          auth = this.app.auth();
          _context10.next = 3;
          return (0, _effects.call)([auth, auth.signOut]);

        case 3:
        case 'end':
          return _context10.stop();
      }
    }
  }, _marked[9], this);
}

/**
 * @param {function} actionCreator
 */
function syncUser(actionCreator) {
  var channel, _ref2, user;

  return _regenerator2.default.wrap(function syncUser$(_context11) {
    while (1) {
      switch (_context11.prev = _context11.next) {
        case 0:
          _context11.next = 2;
          return (0, _effects.call)(this.auth.createOnAuthStateChangedChannel);

        case 2:
          channel = _context11.sent;

        case 3:
          if (!true) {
            _context11.next = 12;
            break;
          }

          _context11.next = 6;
          return (0, _effects.take)(channel);

        case 6:
          _ref2 = _context11.sent;
          user = _ref2.user;
          _context11.next = 10;
          return (0, _effects.put)(actionCreator(user));

        case 10:
          _context11.next = 3;
          break;

        case 12:
        case 'end':
          return _context11.stop();
      }
    }
  }, _marked[10], this);
}

/**
 * Creates channel that will subscribe to changes
 * to the user's sign-in state.
 *
 * @returns {eventChannel} onAuthStateChangedChannel
 */
function createOnAuthStateChangedChannel() {
  var auth = this.app.auth();
  var channel = (0, _reduxSaga.eventChannel)(function (emit) {
    return auth.onAuthStateChanged(function (user) {
      return emit({ user: user });
    });
  });
  return channel;
}

/**
 * Returns the currently signed-in user (or null).
 *
 * @returns {firebase.User} user
 */
function currentUser() {
  return this.app.auth().currentUser;
}

exports.default = {
  syncUser: syncUser,
  createOnAuthStateChangedChannel: createOnAuthStateChangedChannel,
  signOut: signOut,
  signInWithEmailAndPassword: signInWithEmailAndPassword,
  signInAnonymously: signInAnonymously,
  signInWithCustomToken: signInWithCustomToken,
  signInWithPopup: signInWithPopup,
  signInWithGoogle: signInWithGoogle,
  signInWithFacebook: signInWithFacebook,
  signInWithTwitter: signInWithTwitter,
  signInWithGithub: signInWithGithub,
  createUserWithEmailAndPassword: createUserWithEmailAndPassword,
  currentUser: currentUser
};