'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _effects = require('redux-saga/effects');

var _lodash = require('lodash');

var _actions = require('../actions');

var _actions2 = _interopRequireDefault(_actions);

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var createSignInWithProviderSaga = function createSignInWithProviderSaga(firebaseSagas, method, scopes, customParameters) {
  return _regenerator2.default.mark(function signInWithProviderSaga() {
    return _regenerator2.default.wrap(function signInWithProviderSaga$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.prev = 0;
            _context.next = 3;
            return (0, _effects.call)(firebaseSagas.auth[method], scopes, customParameters);

          case 3:
            _context.next = 9;
            break;

          case 5:
            _context.prev = 5;
            _context.t0 = _context['catch'](0);
            _context.next = 9;
            return (0, _effects.put)(_actions.authActions.authFailure(_context.t0));

          case 9:
          case 'end':
            return _context.stop();
        }
      }
    }, signInWithProviderSaga, this, [[0, 5]]);
  });
};

var createSignInSagas = function createSignInSagas(firebaseSagas, options) {
  return options.signInMethods.map(function (method) {
    switch (method.type) {
      case _constants.signInMethods.ANONYMOUSLY:
        return {
          type: _actions2.default.SIGNIN_ANONYMOUSLY,
          saga: _regenerator2.default.mark(function signInAnonymously() {
            return _regenerator2.default.wrap(function signInAnonymously$(_context2) {
              while (1) {
                switch (_context2.prev = _context2.next) {
                  case 0:
                    _context2.prev = 0;
                    _context2.next = 3;
                    return (0, _effects.call)(firebaseSagas.auth.signInAnonymously);

                  case 3:
                    _context2.next = 9;
                    break;

                  case 5:
                    _context2.prev = 5;
                    _context2.t0 = _context2['catch'](0);
                    _context2.next = 9;
                    return (0, _effects.put)(_actions.authActions.authFailure(_context2.t0));

                  case 9:
                  case 'end':
                    return _context2.stop();
                }
              }
            }, signInAnonymously, this, [[0, 5]]);
          })
        };

      case _constants.signInMethods.EMAIL_AND_PASSWORD:
        return {
          type: _actions2.default.SIGNIN_WITH_EMAIL_AND_PASSWORD,
          saga: _regenerator2.default.mark(function signInWithEmailAndPasswordSaga(action) {
            var _action$payload, email, password;

            return _regenerator2.default.wrap(function signInWithEmailAndPasswordSaga$(_context3) {
              while (1) {
                switch (_context3.prev = _context3.next) {
                  case 0:
                    _context3.prev = 0;
                    _action$payload = action.payload, email = _action$payload.email, password = _action$payload.password;
                    _context3.next = 4;
                    return (0, _effects.call)(firebaseSagas.auth.signInWithEmailAndPassword, email, password);

                  case 4:
                    _context3.next = 10;
                    break;

                  case 6:
                    _context3.prev = 6;
                    _context3.t0 = _context3['catch'](0);
                    _context3.next = 10;
                    return (0, _effects.put)(_actions.authActions.authFailure(_context3.t0));

                  case 10:
                  case 'end':
                    return _context3.stop();
                }
              }
            }, signInWithEmailAndPasswordSaga, this, [[0, 6]]);
          })
        };

      case _constants.signInMethods.GOOGLE:
      case _constants.signInMethods.FACEBOOK:
      case _constants.signInMethods.TWITTER:
      case _constants.signInMethods.GITHUB:
        return {
          type: _actions2.default.SIGNIN_WITH_GOOGLE,
          saga: createSignInWithProviderSaga(firebaseSagas, method.type, method.scopes, method.customParameters)
        };

      default:
        {
          throw new Error('Unknown signInMethod!');
        }
    }
  });
};

var createSignOutSaga = function createSignOutSaga(firebaseSagas) {
  return _regenerator2.default.mark(function signOutSaga() {
    return _regenerator2.default.wrap(function signOutSaga$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.prev = 0;
            _context4.next = 3;
            return (0, _effects.call)(firebaseSagas.auth.signOut);

          case 3:
            _context4.next = 9;
            break;

          case 5:
            _context4.prev = 5;
            _context4.t0 = _context4['catch'](0);
            _context4.next = 9;
            return (0, _effects.put)(_actions.authActions.authFailure(_context4.t0));

          case 9:
          case 'end':
            return _context4.stop();
        }
      }
    }, signOutSaga, this, [[0, 5]]);
  });
};

var createAuthChangedSaga = function createAuthChangedSaga(options) {
  return _regenerator2.default.mark(function authChangedSaga(action) {
    var user;
    return _regenerator2.default.wrap(function authChangedSaga$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            user = action.payload;

            if (!(user && (0, _lodash.has)(options, 'onSignInSuccess'))) {
              _context5.next = 6;
              break;
            }

            _context5.next = 4;
            return options.onSignInSuccess(user);

          case 4:
            _context5.next = 9;
            break;

          case 6:
            if (!(0, _lodash.has)(options, 'onSignOutSuccess')) {
              _context5.next = 9;
              break;
            }

            _context5.next = 9;
            return options.onSignOutSuccess();

          case 9:
          case 'end':
            return _context5.stop();
        }
      }
    }, authChangedSaga, this);
  });
};

var createSyncUserSaga = function createSyncUserSaga(firebaseSagas) {
  return _regenerator2.default.mark(function syncUserSaga() {
    return _regenerator2.default.wrap(function syncUserSaga$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            _context6.next = 2;
            return firebaseSagas.auth.syncUser(_actions.authActions.authChanged);

          case 2:
          case 'end':
            return _context6.stop();
        }
      }
    }, syncUserSaga, this);
  });
};

/**
 * Creates a AuthSaga
 * @function
 * @param {FirebaseSagas} FirebaseSagas
 * @param {object} options
 * @return {generator} authSaga
 * @example
 * import { createAuthSaga } from 'firebase-sagas';
 *
 * ...
 *
 * const authSaga = createAuthSaga(firebaseSagas, {
 *  signInMethods: [
 *    { type: 'signInWithEmailAndPassword' },
 *    { type: 'signInWithGoogle' },
 *   ],
 *  onSignInSuccess: function* onSignInSuccess() {
 *    yield put(push('/Todo'));
 *  },
 *  onSignOutSuccess: function* onSignOutSuccess() {
 *    yield put(push('/'));
 *  },
 * });
 */
var createAuthSaga = function createAuthSaga(firebaseSagas, options) {
  return _regenerator2.default.mark(function authSaga() {
    var syncUserSaga, signInSagas, sagas, signOutSaga, authChangedSaga;
    return _regenerator2.default.wrap(function authSaga$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            _context7.next = 2;
            return (0, _effects.put)(_actions.authActions.authInit(options));

          case 2:
            syncUserSaga = createSyncUserSaga(firebaseSagas);
            _context7.next = 5;
            return (0, _effects.fork)(syncUserSaga);

          case 5:
            signInSagas = createSignInSagas(firebaseSagas, options);
            sagas = signInSagas.map(function (signInSaga) {
              return (0, _effects.takeEvery)(signInSaga.type, signInSaga.saga);
            });
            signOutSaga = createSignOutSaga(firebaseSagas);

            sagas.push((0, _effects.takeEvery)(_actions2.default.SIGNOUT, signOutSaga));
            authChangedSaga = createAuthChangedSaga(options);

            sagas.push((0, _effects.takeEvery)(_actions2.default.AUTH_CHANGED, authChangedSaga));

            _context7.next = 13;
            return (0, _effects.all)(sagas);

          case 13:
          case 'end':
            return _context7.stop();
        }
      }
    }, authSaga, this);
  });
};

exports.default = createAuthSaga;