"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Query = undefined;

var _assign = require("babel-runtime/core-js/object/assign");

var _assign2 = _interopRequireDefault(_assign);

var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require("babel-runtime/helpers/createClass");

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @class Query
 * @classdesc
 * Query
 */
var Query = exports.Query = function () {
  /**
   * constructor
   */
  function Query() {
    (0, _classCallCheck3.default)(this, Query);

    this.query = {};
  }

  /**
   * Creates a Query that includes children that match the specified value.
   * @param value The value to match for. The argument type depends on which orderBy*() function was used in this query. Specify a value that matches the orderBy*() type. When used in combination with orderByKey(), the value must be a string.
   * @param {optional} key  The child key to start at, among the children with the previously specified priority. This argument is only allowed if ordering by child, value, or priority.
   * @method Query#equalTo
   */


  (0, _createClass3.default)(Query, [{
    key: "equalTo",
    value: function equalTo(value, key) {
      this.query = (0, _assign2.default)(this.query, { equalTo: { needParams: true, params: { value: value, key: key } } });
      return this;
    }

    /**
     * Creates a Query with the specified starting point.
     *
     * @param value The value to start at. The argument type depends on which orderBy*() function was used in this query. Specify a value that matches the orderBy*() type. When used in combination with orderByKey(), the value must be a string.
     * @param {optional} key The child key to start at. This argument is only allowed if ordering by child, value, or priority.
     * @method Query#startAt
     */

  }, {
    key: "startAt",
    value: function startAt(value, key) {
      this.query = (0, _assign2.default)(this.query, { startAt: { needParams: true, params: { value: value, key: key } } });
      return this;
    }

    /**
     * Creates a Query with the specified ending point.
     *
     * @param value The value to end at. The argument type depends on which orderBy*() function was used in this query. Specify a value that matches the orderBy*() type. When used in combination with orderByKey(), the value must be a string.
     * @param {optional} key
     * @method Query#endAt
     */

  }, {
    key: "endAt",
    value: function endAt(value, key) {
      this.query = (0, _assign2.default)(this.query, { endAt: { needParams: true, params: { value: value, key: key } } });
      return this;
    }

    /**
     * Generates a new Query object limited to the last specific number of children.
     *
     * @param limit The maximum number of nodes to include in this query.
     * @method Query#limitToLast
     */

  }, {
    key: "limitToLast",
    value: function limitToLast(limit) {
      this.query = (0, _assign2.default)(this.query, { limitToLast: { needParams: true, params: { limit: limit } } });
      return this;
    }

    /**
     * Generates a new Query limited to the first specific number of children.
     *
     * @param limit The maximum number of nodes to include in this query.
     * @method Query#limitToFirst
     */

  }, {
    key: "limitToFirst",
    value: function limitToFirst(limit) {
      this.query = (0, _assign2.default)(this.query, { limitToFirst: { needParams: true, params: { limit: limit } } });
      return this;
    }

    /**
     * Generates a new Query object ordered by value.
     * @method Query#orderByValue
     */

  }, {
    key: "orderByValue",
    value: function orderByValue() {
      this.query = (0, _assign2.default)(this.query, { orderByValue: { needParams: false } });
      return this;
    }

    /**
     * Generates a new Query object ordered by priority.
     * @method Query#orderByPriority
     */

  }, {
    key: "orderByPriority",
    value: function orderByPriority() {
      this.query = (0, _assign2.default)(this.query, { orderByPriority: { needParams: false } });
      return this;
    }

    /**
     * Generates a new Query object ordered by the specified child key.
     *
     * @param path
     * @method Query#orderByChild
     */

  }, {
    key: "orderByChild",
    value: function orderByChild(path) {
      this.query = (0, _assign2.default)(this.query, { orderByChild: { needParams: true, params: { path: path } } });
      return this;
    }

    /**
     * Generates a new Query object ordered by key.
     * @method Query#orderByKey
     */

  }, {
    key: "orderByKey",
    value: function orderByKey() {
      this.query = (0, _assign2.default)(this.query, { orderByKey: { needParams: false } });
      return this;
    }

    /**
     * Resets the query
     * @method Query#reset
     */

  }, {
    key: "reset",
    value: function reset() {
      this.query = {};
    }

    /**
     * Return the query as JSON-String
     * @method Query#toJSON
     */

  }, {
    key: "toJSON",
    value: function toJSON() {
      return this.query;
    }
  }]);
  return Query;
}();

/**
 * Creates a new Query
 * @function
 * @return {Query} query
 * @example
 * import { createQuery } from 'firebase-sagas';
 *
 * const query = createQuery().startAt(10).endAt(15).orderByValue();
 */


var createQuery = function createQuery() {
  return new Query();
};

exports.default = createQuery;