'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _reduxSaga = require('redux-saga');

var _effects = require('redux-saga/effects');

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _marked = [once, push, update, set, remove, on].map(_regenerator2.default.mark); /**
                                                                                      * A module for firebaseSagas.database
                                                                                      * @module database
                                                                                      */


/**
 * @private
 * @param snapshot
 * @return array
 */
var toArray = function toArray(snapshot) {
  var array = [];
  snapshot.forEach(function (childSnapshot) {
    var val = childSnapshot.val();
    val.key = childSnapshot.key;
    array.push(val);
  });
  return array;
};

/**
 * @private
 * @param ref
 * @param query
 * @return ref
 */
var extendRefWithQuery = function extendRefWithQuery(ref, query) {
  var tmpRef = ref;
  (0, _keys2.default)(query).forEach(function (key) {
    if (query[key].needParams) {
      switch (key) {
        case 'equalTo':
        case 'startAt':
        case 'endAt':
          {
            var params = query[key].params;
            var para1 = params[(0, _keys2.default)(params)[0]];
            var para2 = params[(0, _keys2.default)(params)[1]];
            tmpRef = tmpRef[key](para1, para2);
            break;
          }

        case 'limitToLast':
        case 'limitToFirst':
        case 'orderByChild':
          {
            var _params = query[key].params;
            var _para = _params[(0, _keys2.default)(_params)[0]];
            tmpRef = tmpRef[key](_para);
            break;
          }

        default:
          throw new Error('Unknown query!');
      }
    } else {
      tmpRef = tmpRef[key]();
    }
  });
  return tmpRef;
};

/**
 * Retrieve data from database just once without subscribing or listening for data changes.
 *
 * @param {string} path
 * @param {string} eventType
 * @param {object} options
 * @returns {*|any}
 */
function once(path, eventType) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
  var ref, snapshot;
  return _regenerator2.default.wrap(function once$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          ref = this.app.database().ref(path);


          if (options && options.query) {
            ref = extendRefWithQuery(ref, options.query);
          }

          _context.next = 4;
          return (0, _effects.call)([ref, ref.once], eventType);

        case 4:
          snapshot = _context.sent;

          if (!(options && options.asArray === true)) {
            _context.next = 7;
            break;
          }

          return _context.abrupt('return', toArray(snapshot));

        case 7:
          return _context.abrupt('return', snapshot.val());

        case 8:
        case 'end':
          return _context.stop();
      }
    }
  }, _marked[0], this);
}

/**
 * Generates a new child location using a unique key and returns its Reference
 *
 * @param {string} path
 * @param value
 * @returns {*|any}
 */
function push(path, value) {
  var ref, result;
  return _regenerator2.default.wrap(function push$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          ref = this.app.database().ref(path);
          _context2.next = 3;
          return (0, _effects.call)([ref, ref.push], value);

        case 3:
          result = _context2.sent;
          return _context2.abrupt('return', result);

        case 5:
        case 'end':
          return _context2.stop();
      }
    }
  }, _marked[1], this);
}

/**
 * Writes multiple values to the Database at once.
 *
 * @param {string} path
 * @param values
 */
function update(path, values) {
  var ref;
  return _regenerator2.default.wrap(function update$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          ref = this.app.database().ref(path);
          _context3.next = 3;
          return (0, _effects.call)([ref, ref.update], values);

        case 3:
        case 'end':
          return _context3.stop();
      }
    }
  }, _marked[2], this);
}

/**
 * Writes data to this Database location.
 *
 * @param {string} path
 * @param value
 */
function set(path, value) {
  var ref;
  return _regenerator2.default.wrap(function set$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          ref = this.app.database().ref(path);
          _context4.next = 3;
          return (0, _effects.call)([ref, ref.set], value);

        case 3:
        case 'end':
          return _context4.stop();
      }
    }
  }, _marked[3], this);
}

/**
 * Removes the data at this Database location.
 *
 * @param {string} path
 */
function remove(path) {
  var ref;
  return _regenerator2.default.wrap(function remove$(_context5) {
    while (1) {
      switch (_context5.prev = _context5.next) {
        case 0:
          ref = this.app.database().ref(path);
          _context5.next = 3;
          return (0, _effects.call)([ref, ref.remove]);

        case 3:
        case 'end':
          return _context5.stop();
      }
    }
  }, _marked[4], this);
}

/**
 * @param {string} path
 * @param {string} eventType
 * @param {function} actionCreator
 * @param {object} options
 */
function on(path, eventType, actionCreator) {
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;

  var channel, _ref, dataSnapshot, _ref2, childSnapshot, prevChildKey, _ref3, oldChildSnapshot;

  return _regenerator2.default.wrap(function on$(_context6) {
    while (1) {
      switch (_context6.prev = _context6.next) {
        case 0:
          _context6.next = 2;
          return (0, _effects.call)(this.database.createOnEventChannel, path, eventType, options);

        case 2:
          channel = _context6.sent;

        case 3:
          if (!true) {
            _context6.next = 47;
            break;
          }

          _context6.t0 = eventType;
          _context6.next = _context6.t0 === _constants.eventTypes.VALUE ? 7 : _context6.t0 === _constants.eventTypes.CHILD_ADDED ? 19 : _context6.t0 === _constants.eventTypes.CHILD_CHANGED ? 19 : _context6.t0 === _constants.eventTypes.CHILD_MOVED ? 19 : _context6.t0 === _constants.eventTypes.CHILD_REMOVED ? 32 : 44;
          break;

        case 7:
          _context6.next = 9;
          return (0, _effects.take)(channel);

        case 9:
          _ref = _context6.sent;
          dataSnapshot = _ref.dataSnapshot;

          if (!(options && options.asArray === true)) {
            _context6.next = 16;
            break;
          }

          _context6.next = 14;
          return (0, _effects.put)(actionCreator(toArray(dataSnapshot)));

        case 14:
          _context6.next = 18;
          break;

        case 16:
          _context6.next = 18;
          return (0, _effects.put)(actionCreator(dataSnapshot.val()));

        case 18:
          return _context6.abrupt('break', 45);

        case 19:
          _context6.next = 21;
          return (0, _effects.take)(channel);

        case 21:
          _ref2 = _context6.sent;
          childSnapshot = _ref2.childSnapshot;
          prevChildKey = _ref2.prevChildKey;

          if (!(options && options.asArray === true)) {
            _context6.next = 29;
            break;
          }

          _context6.next = 27;
          return (0, _effects.put)(actionCreator(toArray(childSnapshot), prevChildKey));

        case 27:
          _context6.next = 31;
          break;

        case 29:
          _context6.next = 31;
          return (0, _effects.put)(actionCreator(childSnapshot.val(), prevChildKey));

        case 31:
          return _context6.abrupt('break', 45);

        case 32:
          _context6.next = 34;
          return (0, _effects.take)(channel);

        case 34:
          _ref3 = _context6.sent;
          oldChildSnapshot = _ref3.oldChildSnapshot;

          if (!(options && options.asArray === true)) {
            _context6.next = 41;
            break;
          }

          _context6.next = 39;
          return (0, _effects.put)(actionCreator(toArray(oldChildSnapshot)));

        case 39:
          _context6.next = 43;
          break;

        case 41:
          _context6.next = 43;
          return (0, _effects.put)(actionCreator(oldChildSnapshot.val()));

        case 43:
          return _context6.abrupt('break', 45);

        case 44:
          throw new Error('sync: Unknown eventType');

        case 45:
          _context6.next = 3;
          break;

        case 47:
        case 'end':
          return _context6.stop();
      }
    }
  }, _marked[5], this);
}

/**
 * @param {string} path
 * @param {string} eventType
 * @returns {eventChannel} onEventChannel
 */
function createOnEventChannel(path, eventType) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

  // if (this.onEventChannel[path][eventType]) {
  //   return this.onEventChannel[path][eventType];
  // }
  var ref = this.app.database().ref(path);

  if (options && options.query) {
    ref = extendRefWithQuery(ref, options.query);
  }

  switch (eventType) {
    case _constants.eventTypes.VALUE:
      // Handle a new value
      return (0, _reduxSaga.eventChannel)(function (emit) {
        var callback = ref.on(eventType, function (dataSnapshot) {
          return emit({ dataSnapshot: dataSnapshot });
        });
        return function () {
          return ref.off(eventType, callback);
        }; // The subscriber must return an unsubscribe function
      });

    case _constants.eventTypes.CHILD_ADDED: // Handle a new child
    case _constants.eventTypes.CHILD_CHANGED: // Handle child data changes
    case _constants.eventTypes.CHILD_MOVED:
      // Handle child ordering changes
      return (0, _reduxSaga.eventChannel)(function (emit) {
        var callback = ref.on(eventType, function (childSnapshot, prevChildKey) {
          return emit({ childSnapshot: childSnapshot, prevChildKey: prevChildKey });
        });
        return function () {
          return ref.off(eventType, callback);
        }; // The subscriber must return an unsubscribe function
      });

    case _constants.eventTypes.CHILD_REMOVED:
      // Handle child removal
      return (0, _reduxSaga.eventChannel)(function (emit) {
        var callback = ref.on(eventType, function (oldChildSnapshot) {
          return emit({ oldChildSnapshot: oldChildSnapshot });
        });
        return function () {
          return ref.off(eventType, callback);
        }; // The subscriber must return an unsubscribe function
      });

    default:
      throw new Error('createOnEventChannel: Unknown eventType');
  }
}

exports.default = {
  once: once,
  push: push,
  update: update,
  set: set,
  remove: remove,
  on: on,
  createOnEventChannel: createOnEventChannel
};